@@ -1,5 +1,5 @@
 /******************************************************************************
-* Copyright 1996-2014 United States Government as represented by the
+* Copyright 1996-2016 United States Government as represented by the
 * Administrator of the National Aeronautics and Space Administration.
 * All Rights Reserved.
 ******************************************************************************/
@@ -57,17 +57,26 @@
 *                               CDFinsertVarRecorsByVarName.
 *   V3.12 28-May-14, M Liu      Added function items for record deletion:
 *                               r/zVAR__RECORDS_RENUMBER_.
+*   V3.13 20-Oct-16, M Liu      Added a new set CDFread functions with CDFdata
+*                               type. Moved a few functions from cdfdist.h to
+*                               here.
+*   V3.14 10-Jan-18, M Liu      Added ARM and IA64 openVMS encoding.
+*   V3.15 10-Apr-21, M Liu      Added a few functions to handle UTF-8 encoding
+*                               characters.
 ******************************************************************************/
 
 #if !defined(CDFh_INCLUDEd__)
 #define CDFh_INCLUDEd__
 
+#include <wchar.h>
+
 /******************************************************************************
 * CDF defined types
 ******************************************************************************/
 
 typedef void *CDFid;
 typedef long CDFstatus;
+typedef void *CDFdata;
 
 /******************************************************************************
 * CDF defined variables
@@ -95,7 +104,7 @@ static double *TT2000NULL = 0;
 #define CDF_ATTR_NAME_LEN256    256
 
 #define CDF_COPYRIGHT_LEN       256
-#define CDF_STATUSTEXT_LEN      120
+#define CDF_STATUSTEXT_LEN      200
 #define CDF_PATHNAME_LEN        512
 
 #define EPOCH_STRING_LEN	24
@@ -114,6 +123,7 @@ static double *TT2000NULL = 0;
 #define TT2000_1_STRING_LEN     19
 #define TT2000_2_STRING_LEN     14
 #define TT2000_3_STRING_LEN     29
+#define TT2000_4_STRING_LEN     30
 
 #define EPOCHx_STRING_MAX	50
 #define EPOCHx_FORMAT_MAX	68
@@ -159,7 +169,12 @@ static double *TT2000NULL = 0;
 #define ALPHAOSF1_ENCODING      13L
 #define ALPHAVMSd_ENCODING      14L
 #define ALPHAVMSg_ENCODING      15L
-#define ALPHAVMSi_ENCODING	16L
+#define ALPHAVMSi_ENCODING      16L
+#define ARM_LITTLE_ENCODING     17L
+#define ARM_BIG_ENCODING        18L
+#define IA64VMSi_ENCODING       19L
+#define IA64VMSd_ENCODING       20L
+#define IA64VMSg_ENCODING       21L
 
 /******************************************************************************
 * Decodings.
@@ -181,7 +196,12 @@ static double *TT2000NULL = 0;
 #define ALPHAOSF1_DECODING      ALPHAOSF1_ENCODING
 #define ALPHAVMSd_DECODING      ALPHAVMSd_ENCODING
 #define ALPHAVMSg_DECODING      ALPHAVMSg_ENCODING
-#define ALPHAVMSi_DECODING	ALPHAVMSi_ENCODING
+#define ALPHAVMSi_DECODING      ALPHAVMSi_ENCODING
+#define ARM_LITTLE_DECODING     ARM_LITTLE_ENCODING
+#define ARM_BIG_DECODING        ARM_BIG_ENCODING
+#define IA64VMSi_DECODING       IA64VMSi_ENCODING
+#define IA64VMSd_DECODING       IA64VMSd_ENCODING
+#define IA64VMSg_DECODING       IA64VMSg_ENCODING
 
 /******************************************************************************
 * Variance flags
@@ -286,7 +306,10 @@ static double *TT2000NULL = 0;
 						   hasn't been selected yet. */
 
 #define ILLEGAL_EPOCH_VALUE	(-1.0)
-#define ILLEGAL_TT2000_VALUE    (-9223372036854775805LL)
+#define ILLEGAL_TT2000_VALUE    (-9223372036854775805LL) 
+                                             /* as
+                                                1707-09-22T12:12:10.961224195*/
+
 #define FILLED_TT2000_VALUE	(-9223372036854775807LL-1)
 
 /******************************************************************************
@@ -303,6 +326,10 @@ static double *TT2000NULL = 0;
 #define MULTI_FILE_FORMAT		((CDFstatus) 1007)
 #define SOME_ALREADY_ALLOCATED		((CDFstatus) 1008)
 #define PRECEEDING_RECORDS_ALLOCATED	((CDFstatus) 1009)
+#define TT2000_CDF_MAYNEEDUPDATE        ((CDFstatus) 1010)
+#define BLOCKINGFACTOR_TOO_SMALL        ((CDFstatus) 1011)
+#define BLOCKINGFACTOR_TOO_SMALL2       ((CDFstatus) 1012)
+#define STRING_NOT_UTF8_ENCODING        ((CDFstatus) 1013)
 
 #define CDF_OK                          ((CDFstatus) 0)
 
@@ -439,6 +466,12 @@ static double *TT2000NULL = 0;
 #define ZLIB_COMPRESS_ERROR             ((CDFstatus) (-2231))
 #define ZLIB_UNCOMPRESS_ERROR           ((CDFstatus) (-2232))
 #define CANNOT_INSERT_RECORDS           ((CDFstatus) (-2233))
+#define TT2000_USED_OUTDATED_TABLE      ((CDFstatus) (-2234))
+#define BADDATE_LEAPSECOND_UPDATED      ((CDFstatus) (-2235))
+#define FUNCTION_NOT_SUPPORTED          ((CDFstatus) (-2236))
+#define TRY_TO_READ_NONSTRING_DATA      ((CDFstatus) (-2237))
+#define BAD_NUM_STRINGS                 ((CDFstatus) (-2238))
+#define CANNOT_CONVERT_WIDECHAR         ((CDFstatus) (-2239))
 
 /******************************************************************************
 * Functions (for INTERNAL interface).
@@ -628,11 +661,17 @@ static double *TT2000NULL = 0;
 #define CDF_CHECKSUM_           156L
 #define rVAR_RECORDS_RENUMBER_	157L
 #define zVAR_RECORDS_RENUMBER_	158L
+#define CDF_LEAPSECONDLASTUPDATED_ 159L
+#define rENTRY_STRINGSDATA_     160L
+#define zENTRY_STRINGSDATA_     161L
+#define rENTRY_NUMSTRINGS_      162L
+#define zENTRY_NUMSTRINGS_      163L
 
 #define CDFwithSTATS_		200L	/* For CDF internal use only! */
 #define CDF_ACCESS_		201L	/* For CDF internal use only! */
 
 #define TT2000END 		-99999.999
+#define STRINGDELIMITER		"\\N "
 
 /******************************************************************************
 * C interface macros.
@@ -672,6 +711,20 @@ CDFputAttrEntry(id,2,attrNum,entryNum,dataType,numElems,value)
 #define CDFputAttrzEntry(id,attrNum,entryNum,dataType,numElems,value) \
 CDFputAttrEntry(id,3,attrNum,entryNum,dataType,numElems,value)
 
+#define CDFputAttrStrgEntry(id,attrNum,entryNum,string) \
+CDFputAttrStrEntry(id,1,attrNum,entryNum,string)
+#define CDFputAttrStrrEntry(id,attrNum,entryNum,string) \
+CDFputAttrStrEntry(id,2,attrNum,entryNum,string)
+#define CDFputAttrStrzEntry(id,attrNum,entryNum,string) \
+CDFputAttrStrEntry(id,3,attrNum,entryNum,string)
+
+#define CDFputAttrWStrgEntry(id,attrNum,entryNum,string,locale) \
+CDFputAttrWStrEntry(id,1,attrNum,entryNum,string,locale)
+#define CDFputAttrWStrrEntry(id,attrNum,entryNum,string,locale) \
+CDFputAttrWStrEntry(id,2,attrNum,entryNum,string,locale)
+#define CDFputAttrWStrzEntry(id,attrNum,entryNum,string,locale) \
+CDFputAttrWStrEntry(id,3,attrNum,entryNum,string,locale)
+
 #define CDFattrGet(id,attrNum,entryNum,value) \
 CDFgetAttrEntry(id,0,attrNum,entryNum,value)
 #define CDFgetAttrgEntry(id,attrNum,entryNum,value) \
@@ -681,6 +734,30 @@ CDFgetAttrEntry(id,2,attrNum,entryNum,value)
 #define CDFgetAttrzEntry(id,attrNum,entryNum,value) \
 CDFgetAttrEntry(id,3,attrNum,entryNum,value)
 
+#define CDFgetAttrStrgEntry(id,attrNum,entryNum,string) \
+CDFgetAttrStrEntry(id,1,attrNum,entryNum,string)
+#define CDFgetAttrStrrEntry(id,attrNum,entryNum,string) \
+CDFgetAttrStrEntry(id,2,attrNum,entryNum,string)
+#define CDFgetAttrStrzEntry(id,attrNum,entryNum,string) \
+CDFgetAttrStrEntry(id,3,attrNum,entryNum,string)
+
+#define CDFgetAttrWStrgEntry(id,attrNum,entryNum,string,locale) \
+CDFgetAttrWStrEntry(id,1,attrNum,entryNum,string,locale)
+#define CDFgetAttrWStrrEntry(id,attrNum,entryNum,string,locale) \
+CDFgetAttrWStrEntry(id,2,attrNum,entryNum,string,locale)
+#define CDFgetAttrWStrzEntry(id,attrNum,entryNum,string,locale) \
+CDFgetAttrWStrEntry(id,3,attrNum,entryNum,string,locale)
+
+#define CDFgetAttrrEntryStrings(id,attrNum,entryNum,numStrings,strings) \
+CDFgetAttrEntryStrings(id,0,attrNum,entryNum,numStrings,strings)
+#define CDFgetAttrzEntryStrings(id,attrNum,entryNum,numStrings,strings) \
+CDFgetAttrEntryStrings(id,1,attrNum,entryNum,numStrings,strings)
+
+#define CDFputAttrrEntryStrings(id,attrNum,entryNum,numStrings,strings) \
+CDFputAttrEntryStrings(id,0,attrNum,entryNum,numStrings,strings)
+#define CDFputAttrzEntryStrings(id,attrNum,entryNum,numStrings,strings) \
+CDFputAttrEntryStrings(id,1,attrNum,entryNum,numStrings,strings)
+
 #define CDFgetAttrgEntryDataType(id,attrNum,entryNum,dataType) \
 CDFgetAttrEntryDataType(id,1,attrNum,entryNum,dataType)
 #define CDFgetAttrrEntryDataType(id,attrNum,entryNum,dataType) \
@@ -760,6 +837,11 @@ CDFgetAttrEntryNumElements(id,2,attrNum,entryNum,numElems)
 #define CDFgetAttrzEntryNumElements(id,attrNum,entryNum,numElems) \
 CDFgetAttrEntryNumElements(id,3,attrNum,entryNum,numElems)
 
+#define CDFgetAttrrEntryNumStrings(id,attrNum,entryNum,numStrings) \
+CDFgetAttrEntryNumStrings(id,0,attrNum,entryNum,numStrings)
+#define CDFgetAttrzEntryNumStrings(id,attrNum,entryNum,numStrings) \
+CDFgetAttrEntryNumStrings(id,1,attrNum,entryNum,numStrings)
+
 #define CDFgetNumrVars(id,numVars) CDFgetNumVars(id,0,numVars)
 #define CDFgetNumzVars(id,numVars) CDFgetNumVars(id,1,numVars)
 
@@ -1111,6 +1193,11 @@ CDFlib (SELECT_, CDF_, id, \
         GET_, CDF_FORMAT_, format, \
         NULL_)
 
+#define CDFgetLeapSecondLastUpdated(id,lastUpdated) \
+CDFlib (SELECT_, CDF_, id, \
+        GET_, CDF_LEAPSECONDLASTUPDATED_, lastUpdated, \
+        NULL_)
+
 #define CDFgetCompressionInfo(name,cType,cParms,cSize,uSize) \
 CDFlib (GET_, CDF_INFO_, name, cType, cParms, cSize, uSize, \
         NULL_)
@@ -1399,6 +1486,11 @@ CDFlib (SELECT_, CDF_, id, \
         PUT_, CDF_FORMAT_, format, \
         NULL_)
 
+#define CDFsetLeapSecondLastUpdated(id,lastUpdated) \
+CDFlib (SELECT_, CDF_, id, \
+        PUT_, CDF_LEAPSECONDLASTUPDATED_, lastUpdated, \
+        NULL_)
+
 #define CDFsetMajority(id,majority) \
 CDFlib (SELECT_, CDF_, id, \
         PUT_, CDF_MAJORITY_, majority, \
@@ -1583,12 +1675,12 @@ CDFlib (SELECT_, CDF_, id, \
 /******************************************************************************
 * TT2000 macros define'd
 ******************************************************************************/
+#define CDF_TT2000_from_UTC_string parseTT2000
+#define CDF_TT2000_to_UTC_string encodeTT2000
+#define CDF_TT2000_from_UTC_parts computeTT2000
+#define CDF_TT2000_to_UTC_parts breakdownTT2000
 
-#define parseTT2000 CDF_TT2000_from_UTC_string
-#define encodeTT2000 CDF_TT2000_to_UTC_string
-#define computeTT2000 CDF_TT2000_from_UTC_parts
-#define breakdownTT2000 CDF_TT2000_to_UTC_parts
-#define TT2000breakdown CDF_TT2000_to_UTC_parts
+#define TT2000breakdown breakdownTT2000
 
 /******************************************************************************
 * Function prototypes.
@@ -1678,9 +1770,46 @@ VISIBLE_PREFIX CDFstatus CDFputAttrEntry PROTOARGs((
   CDFid id, int grzEntry, long attrNum, long entryNum, long dataType,
   long numElems, void *value
 ));
+VISIBLE_PREFIX CDFstatus CDFputAttrStrEntry PROTOARGs((
+  CDFid id, int grzEntry, long attrNum, long entryNum, char *string
+));
 VISIBLE_PREFIX CDFstatus CDFgetAttrEntry PROTOARGs((
   CDFid id, int grzEntry, long attrNum, long entryNum, void *value
 ));
+VISIBLE_PREFIX CDFstatus CDFgetAttrStrEntry PROTOARGs((
+  CDFid id, int grzEntry, long attrNum, long entryNum, char **string
+));
+//#if defined(WIN32)
+VISIBLE_PREFIX CDFstatus CDFputAttrWStrEntry PROTOARGs((
+  CDFid id, int grzEntry, long attrNum, long entryNum, wchar_t *wstring,
+  char *locale
+));
+VISIBLE_PREFIX CDFstatus CDFgetAttrWStrEntry PROTOARGs((
+  CDFid id, int grzEntry, long attrNum, long entryNum, wchar_t **wstring,
+  char *locale
+));
+//#endif
+VISIBLE_PREFIX CDFstatus CDFgetAttrEntryNumStrings PROTOARGs((
+  CDFid id, int rzEntry, long attrNum, long entryNum, 
+  long *numStrings
+));
+VISIBLE_PREFIX CDFstatus CDFgetAttrEntryStrings PROTOARGs((
+  CDFid id, int rzEntry, long attrNum, long entryNum, 
+  long *numStrings, char ***strings
+));
+VISIBLE_PREFIX CDFstatus CDFputAttrEntryStrings PROTOARGs((
+  CDFid id, int rzEntry, long attrNum, long entryNum, 
+  long numStrings, char **strings
+));
+VISIBLE_PREFIX void CDFdataFree PROTOARGs((CDFdata value));
+VISIBLE_PREFIX CDFstatus CDFreadgAttrEntry PROTOARGs((
+  CDFid id, long attrNum, long entryNum, long *dataType, long *numElms,
+  CDFdata *value
+));
+VISIBLE_PREFIX CDFstatus CDFreadzAttrEntry PROTOARGs((
+  CDFid id, long attrNum, long entryNum, long *dataType, long *numElms,
+  CDFdata *value
+));
 VISIBLE_PREFIX CDFstatus CDFdeleteAttrEntry PROTOARGs((
   CDFid id, int grzEntry, long attrNum, long entryNum
 ));
@@ -1719,6 +1848,24 @@ VISIBLE_PREFIX CDFstatus CDFputVarsRecordDatabyNames PROTOARGs((
 VISIBLE_PREFIX CDFstatus CDFgetVarAllRecordsByVarID PROTOARGs((
   CDFid id, int zVar, long varNum, void *buffer
 ));
+VISIBLE_PREFIX CDFstatus CDFgetzVarSpec PROTOARGs((
+  CDFid id, long varNum, long *dataType, long *numElems,
+  long *numDims, long *dimSizes, long *recVary, long *dimsVary 
+));
+VISIBLE_PREFIX CDFstatus CDFreadzVarPadValue PROTOARGs((
+  CDFid id, long varNum, long *dataType, long *numElms, CDFdata *buffer
+));
+VISIBLE_PREFIX CDFstatus CDFreadzVarAllByVarID PROTOARGs((
+  CDFid id, long varNum, long *numRecs, long *dataType, long *numElems,
+  long *numDims, long *dimSizes, long *recVary, long *dimsVary, 
+  CDFdata *buffer
+));
+VISIBLE_PREFIX CDFstatus CDFreadzVarDataByVarID PROTOARGs((
+  CDFid id, long varNum, long *numRecs, CDFdata *buffer
+));
+VISIBLE_PREFIX CDFstatus CDFreadzVarRangeDataByVarID PROTOARGs((
+  CDFid id, long varNum, long strNum, long stpNum, CDFdata *buffer
+));
 VISIBLE_PREFIX CDFstatus CDFputVarAllRecordsByVarID PROTOARGs((
   CDFid id, int zVar, long varNum, long numRec, void *buffer
 ));
@@ -1733,6 +1880,17 @@ VISIBLE_PREFIX CDFstatus CDFputVarRangeRecordsByVarID PROTOARGs((
 VISIBLE_PREFIX CDFstatus CDFgetVarAllRecordsByVarName PROTOARGs((
   CDFid id, char *varName, void *buffer
 ));
+VISIBLE_PREFIX CDFstatus CDFreadzVarAllByVarName PROTOARGs((
+  CDFid id, char *varName, long *numRecs, long *dataType, long *numElems,
+  long *numDims, long *dimSizes, long *recVary, long *dimsVary, 
+  CDFdata *buffer
+));
+VISIBLE_PREFIX CDFstatus CDFreadzVarDataByVarName PROTOARGs((
+  CDFid id, char *varName, long *numRecs, CDFdata *buffer
+));
+VISIBLE_PREFIX CDFstatus CDFreadzVarRangeDataByVarName PROTOARGs((
+  CDFid id, char *varName, long strNum, long stpNum, CDFdata *buffer
+));
 VISIBLE_PREFIX CDFstatus CDFputVarAllRecordsByVarName PROTOARGs((
   CDFid id, char *varName, long numRecs, void *buffer
 ));
@@ -1771,9 +1929,16 @@ VISIBLE_PREFIX int CDFgetValidateDebug PROTOARGs(());
 VISIBLE_PREFIX void ROWtoCOL PROTOARGs((
   void *iBuffer, void *oBuffer, long numDims, long dimSizes[], long nValueBytes
 ));
+VISIBLE_PREFIX int CDFelemSize PROTOARGs((long));
 VISIBLE_PREFIX void COLtoROW PROTOARGs((
   void *iBuffer, void *oBuffer, long numDims, long dimSizes[], long nValueBytes
 ));
+VISIBLE_PREFIX int CDFstrcmpIgCase PROTOARGs((char *string1, char *string2));
+VISIBLE_PREFIX char *CDFstrstrIgCase PROTOARGs((char *string1, char *string2));
+VISIBLE_PREFIX void StringBreakIntoStrings PROTOARGs((
+char *string, long numElms, long numStrs, char ***strings
+));
+
 #if !defined(__CFM68K__) || defined(__USING_STATIC_LIBS__) || !defined(CFM68KDLL)
 VISIBLE_PREFIX void EPOCHbreakdown PROTOARGs((
   double epoch, long *year, long *month, long *day, long *hour, long *minute,
@@ -1783,31 +1948,45 @@ VISIBLE_PREFIX double computeEPOCH PROTOARGs((
   long year, long month, long day, long hour, long minute, long second,
   long msec
 ));
+VISIBLE_PREFIX double toParseEPOCH PROTOARGs((char *inString));
 VISIBLE_PREFIX double parseEPOCH PROTOARGs((char *inString));
 VISIBLE_PREFIX double parseEPOCH1 PROTOARGs((char *inString));
 VISIBLE_PREFIX double parseEPOCH2 PROTOARGs((char *inString));
 VISIBLE_PREFIX double parseEPOCH3 PROTOARGs((char *inString));
 VISIBLE_PREFIX double parseEPOCH4 PROTOARGs((char *inString));
-VISIBLE_PREFIX void encodeEPOCH PROTOARGs((
+VISIBLE_PREFIX void   toEncodeEPOCH PROTOARGs((
+  double epoch, int style, char *epString
+));
+VISIBLE_PREFIX void   encodeEPOCH PROTOARGs((
   double epoch, char epString[EPOCH_STRING_LEN+1]
 ));
-VISIBLE_PREFIX void encodeEPOCH1 PROTOARGs((
+VISIBLE_PREFIX void   encodeEPOCH1 PROTOARGs((
   double epoch, char epString[EPOCH1_STRING_LEN+1]
 ));
-VISIBLE_PREFIX void encodeEPOCH2 PROTOARGs((
+VISIBLE_PREFIX void   encodeEPOCH2 PROTOARGs((
   double epoch, char epString[EPOCH2_STRING_LEN+1]
 ));
-VISIBLE_PREFIX void encodeEPOCH3 PROTOARGs((
+VISIBLE_PREFIX void   encodeEPOCH3 PROTOARGs((
   double epoch, char epString[EPOCH3_STRING_LEN+1]
 ));
-VISIBLE_PREFIX void encodeEPOCH4 PROTOARGs((
+VISIBLE_PREFIX void   encodeEPOCH4 PROTOARGs((
   double epoch, char epString[EPOCH4_STRING_LEN+1]
 ));
-VISIBLE_PREFIX void encodeEPOCHx PROTOARGs((
+VISIBLE_PREFIX void   encodeEPOCHx PROTOARGs((
   double epoch, char format[EPOCHx_FORMAT_MAX],
   char encoded[EPOCHx_STRING_MAX]
 ));
-VISIBLE_PREFIX void EPOCH16breakdown PROTOARGs((
+VISIBLE_PREFIX void EPOCHtoUnixTime PROTOARGs((
+  double *epoch,
+  double *unitTime,
+  int    numTimes
+));
+VISIBLE_PREFIX void UnixTimetoEPOCH PROTOARGs((
+  double *unitTime,
+  double *epoch,
+  int    numTimes
+));
+VISIBLE_PREFIX void   EPOCH16breakdown PROTOARGs((
   double *epoch, long *year, long *month, long *day, long *hour,
   long *minute, long *second, long *msec, long *usec, long *nsec, long *psec
 ));
@@ -1815,6 +1994,9 @@ VISIBLE_PREFIX double computeEPOCH16 PROTOARGs((
   long year, long month, long day, long hour, long minute, long second,
   long msec, long usec, long nsec, long psec, double *epoch
 ));
+VISIBLE_PREFIX double toParseEPOCH16 PROTOARGs((char *inString,
+  double *epoch
+));
 VISIBLE_PREFIX double parseEPOCH16 PROTOARGs((char *inString,
   double *epoch
 ));
@@ -1830,33 +2012,57 @@ VISIBLE_PREFIX double parseEPOCH16_3 PROTOARGs((char *inStringch,
 VISIBLE_PREFIX double parseEPOCH16_4 PROTOARGs((char *inStringch,
   double *epoch
 ));
-VISIBLE_PREFIX void encodeEPOCH16 PROTOARGs((
+VISIBLE_PREFIX void   toEncodeEPOCH16 PROTOARGs((
+  double *epoch, int style, char *epString
+));
+VISIBLE_PREFIX void   encodeEPOCH16 PROTOARGs((
   double *epoch, char epString[EPOCH16_STRING_LEN+1]
 ));
-VISIBLE_PREFIX void encodeEPOCH16_1 PROTOARGs((
+VISIBLE_PREFIX void   encodeEPOCH16_1 PROTOARGs((
   double *epoch, char epString[EPOCH16_1_STRING_LEN+1]
 ));
-VISIBLE_PREFIX void encodeEPOCH16_2 PROTOARGs((
+VISIBLE_PREFIX void   encodeEPOCH16_2 PROTOARGs((
   double *epoch, char epString[EPOCH16_2_STRING_LEN+1]
 ));
-VISIBLE_PREFIX void encodeEPOCH16_3 PROTOARGs((
+VISIBLE_PREFIX void   encodeEPOCH16_3 PROTOARGs((
   double *epoch, char epString[EPOCH16_3_STRING_LEN+1]
 ));
-VISIBLE_PREFIX void encodeEPOCH16_4 PROTOARGs((
+VISIBLE_PREFIX void   encodeEPOCH16_4 PROTOARGs((
   double *epoch, char epString[EPOCH16_4_STRING_LEN+1]
 ));
-VISIBLE_PREFIX void encodeEPOCH16_x PROTOARGs((
+VISIBLE_PREFIX void   encodeEPOCH16_x PROTOARGs((
   double *epoch, char format[EPOCHx_FORMAT_MAX],
   char encoded[EPOCHx_STRING_MAX]
 ));
+VISIBLE_PREFIX void EPOCH16toUnixTime PROTOARGs((
+  double *epoch,
+  double *unixTime,
+  int    numTimes
+));
+VISIBLE_PREFIX void UnixTimetoEPOCH16 PROTOARGs((
+  double *unixTime,
+  double *epoch,
+  int    numTimes
+));
+VISIBLE_PREFIX int UTF8StrLength PROTOARGs((const char *, int));
+VISIBLE_PREFIX int UTF8CharLength PROTOARGs((const char *));
+VISIBLE_PREFIX int UTF8CharAtOffset PROTOARGs((const char *, int));
+VISIBLE_PREFIX char *UTF8CharAtString PROTOARGs((const char *, int));
+VISIBLE_PREFIX int UTF8CharAtLength PROTOARGs((const char *, int));
+VISIBLE_PREFIX int UTF8StrLengthFromCharAt PROTOARGs((const char *, int));
+VISIBLE_PREFIX int UTF8ByteLengthFromCharAt PROTOARGs((const char *, int));
+VISIBLE_PREFIX int UTF8toUnicode PROTOARGs((int *, const char *, int));
+VISIBLE_PREFIX int UnicodetoUTF8 PROTOARGs((char *, int *, int));
+VISIBLE_PREFIX unsigned char *UTF8Check PROTOARGs((unsigned char *, int, int *));
+VISIBLE_PREFIX int IsUTF8Char(const char *s);
+
 /******************************************************************************
 * A new set of functions to handle CDF_TIME_TT2000 time type.
 ******************************************************************************/
-
-VISIBLE_PREFIX void CDF_TT2000_to_UTC_parts PROTOARGs((
+VISIBLE_PREFIX void breakdownTT2000 PROTOARGs((
   long long tt2000, double *year, double *month, double *day, ...
 ));
-VISIBLE_PREFIX long long CDF_TT2000_from_UTC_parts PROTOARGs((
+VISIBLE_PREFIX long long computeTT2000 PROTOARGs((
   double year, double month, double day, ...
 ));
 VISIBLE_PREFIX double CDF_TT2000_to_UTC_EPOCH PROTOARGs((
@@ -1871,12 +2077,45 @@ VISIBLE_PREFIX double CDF_TT2000_to_UTC_EPOCH16 PROTOARGs((
 VISIBLE_PREFIX long long CDF_TT2000_from_UTC_EPOCH16 PROTOARGs((
   double *epoch16
 ));
-VISIBLE_PREFIX void CDF_TT2000_to_UTC_string PROTOARGs((
+VISIBLE_PREFIX void toEncodeTT2000 PROTOARGs((
+  long long time, int style, char *string
+));
+VISIBLE_PREFIX void encodeTT2000 PROTOARGs((
   long long time, char *string, ...
 ));
-VISIBLE_PREFIX long long CDF_TT2000_from_UTC_string PROTOARGs((
+VISIBLE_PREFIX long long toParseTT2000 PROTOARGs((
   char *string
 ));
+VISIBLE_PREFIX long long parseTT2000 PROTOARGs((
+  char *string
+));
+VISIBLE_PREFIX long long computeTT2000withBasedLeapDay PROTOARGs((
+  long yy, long mm, long dd, long hh, long mn, long ss, long ms,
+  long us, long ns, int yymmdd
+));
+VISIBLE_PREFIX void breakdownTT2000withBasedLeapDay PROTOARGs((
+  long long tt2000, int yymmdd, 
+  long *yy, long *mm, long *dd, long *hh, long *mn, long *ss,
+  long *ms, long *us, long *ns
+));
+VISIBLE_PREFIX long long parseTT2000withBasedLeapDay PROTOARGs((
+  char *tt2000, int yymmdd
+));
+VISIBLE_PREFIX void encodeTT2000withBasedLeapDay PROTOARGs((
+  long long tt2000,
+  int yymmdd,
+  char *string
+));
+VISIBLE_PREFIX void TT2000toUnixTime PROTOARGs((
+  long long *epoch,
+  double *unixTime,
+  int    numTimes
+));
+VISIBLE_PREFIX void UnixTimetoTT2000 PROTOARGs((
+  double *unixTime,
+  long long *epoch,
+  int    numTimes
+));
 VISIBLE_PREFIX void CDFClearLeapSecondsTable PROTOARGs(());
 VISIBLE_PREFIX void CDFfillLeapSecondsTable PROTOARGs((
   double **table
@@ -1894,6 +2133,12 @@ VISIBLE_PREFIX void CDFgetLastDateinLeapSecondsTable PROTOARGs((
 VISIBLE_PREFIX char *CDFgetLeapSecondsTableEnvVar PROTOARGs(());
 VISIBLE_PREFIX int CDFgetLeapSecondsTableStatus PROTOARGs(());
 #endif
+VISIBLE_PREFIX void CDF_Free_String PROTOARGs((
+  long numStrings, char **strings
+));
+VISIBLE_PREFIX long FindNumberOfItems PROTOARGs((
+  char *string, long numElems, char *delimiter
+));
 #if defined(__cplusplus)
 }
 #endif
