@@ -4,8 +4,8 @@ use strict;
 use Math::BigInt;
 use Data::Dumper;
 
-BEGIN { unshift @INC,'/home/liu/PerlCDF38_0/blib/arch',
-                     '/home/liu/PerlCDF38_0/blib/lib'; }
+BEGIN { unshift @INC,$ENV{"HOME"}.'/PerlCDF38_1/blib/arch',
+                     $ENV{"HOME"}.'/PerlCDF38_1/blib/lib'; }
 use CDF;
 
 my $id;
