@@ -37,20 +37,20 @@
 use strict;
 use Math::BigInt;
 
-BEGIN { unshift @INC,'/home/liu/PerlCDF38_0/blib/arch',
-                     '/home/liu/PerlCDF38_0/blib/lib'; }
+BEGIN { unshift @INC,$ENV{"HOME"}.'/PerlCDF38_1/blib/arch',
+                     $ENV{"HOME"}.'/PerlCDF38_1/blib/lib'; }
 use CDF;
 
 my $N_DIMS = 2;
 my $DIM_0_SIZE = 2;
 my $DIM_1_SIZE = 3;
 
-my $encoding = &NETWORK_ENCODING;
-my $actual_encoding = &NETWORK_ENCODING;
-my $majority = &ROW_MAJOR;
+my $encoding = NETWORK_ENCODING;
+my $actual_encoding = NETWORK_ENCODING;
+my $majority = ROW_MAJOR;
 my $numDims = $N_DIMS;
 my @dimSizes = ( $DIM_0_SIZE, $DIM_1_SIZE );
-my $varDataType = &CDF_INT2;
+my $varDataType = CDF_INT2;
 my $varNumElements = 1 ;
 my @varValues = (1,2,3,4,5,6);
 my $recNum = 0 ;
@@ -60,15 +60,15 @@ my $recInterval = 1 ;
 my @counts = ( $DIM_0_SIZE, $DIM_1_SIZE );
 my @intervals = ( 1, 1 );
 my $entryNum = 2 ;
-my $attrScope = &GLOBAL_SCOPE ;
-my $entryDataType = &CDF_INT2 ;
-my $entryDataTypeNew = &CDF_UINT2 ;
+my $attrScope = GLOBAL_SCOPE ;
+my $entryDataType = CDF_INT2 ;
+my $entryDataTypeNew = CDF_UINT2 ;
 my $entryNumElems = 1 ;
 my $entry1NumElems = 6 ;
 my $entryValue = 1 ;
 my @entry1Values = (1,2,3,4,5,6);
-my $varRecVariance = &VARY ;
-my @varDimVariances = ( &VARY, &VARY );
+my $varRecVariance = VARY ;
+my @varDimVariances = ( VARY, VARY );
 my $varName = "VAR1a";
 my $new_varName = "VAR1b";
 my $attrName = "ATTR1";
@@ -90,16 +90,16 @@ print "\nTesting Perl-CDF Standard/C interface\n\n";
 
 my $id; my $status;
 $status = CDF::CDFcreate("YEST",$numDims,\@dimSizes,$encoding,$majority,\$id);
-if ($status < &CDF_OK) {
-  if ($status == &CDF_EXISTS) {
+if ($status < CDF_OK) {
+  if ($status == CDF_EXISTS) {
     $status = CDF::CDFopen("YEST", \$id);
-    QuitCDF("1.0", $status) if ($status < &CDF_OK) ;
+    QuitCDF("1.0", $status) if ($status < CDF_OK) ;
 
     $status = CDF::CDFdelete($id);
-    QuitCDF ("1.1", $status) if ($status < &CDF_OK) ;
+    QuitCDF ("1.1", $status) if ($status < CDF_OK) ;
     $status = CDF::CDFcreate("YEST",$numDims,\@dimSizes,$encoding,$majority,
                                 \$id);
-    QuitCDF ("1.2", $status) if ($status < &CDF_OK) ;
+    QuitCDF ("1.2", $status) if ($status < CDF_OK) ;
   }
   else {
     QuitCDF ("1.3", $status);
@@ -117,7 +117,7 @@ my $varNum_out;
 $status = CDF::CDFvarCreate($id,$varName,$varDataType,$varNumElements,
 			      $varRecVariance,\@varDimVariances,
 			      \$varNum_out); 
-QuitCDF ("2.0a", $status) if ($status < &CDF_OK) ;
+QuitCDF ("2.0a", $status) if ($status < CDF_OK) ;
 
 #############################################################################
 # PUT to variables.
@@ -130,7 +130,7 @@ for ($x0 = 0; $x0 < $DIM_0_SIZE; $x0++) {
       $indices[1] = $x1;
       $temp1 = $varValues[$x0*$DIM_1_SIZE + $x1];
       $status = CDF::CDFvarPut($id,$var_num,$recNum,\@indices,\$temp1);
-      QuitCDF ("8.1", $status) if ($status < &CDF_OK) ;
+      QuitCDF ("8.1", $status) if ($status < CDF_OK) ;
 
    }
 }
@@ -146,7 +146,7 @@ for ($x0 = 0; $x0 < $DIM_0_SIZE; $x0++) {
       $indices[1] = $x1;
       $status = CDF::CDFvarGet($id,$var_num,$recNum,\@indices,
 			         \$varValue_out);
-      QuitCDF ("9.0", $status) if ($status < &CDF_OK) ;
+      QuitCDF ("9.0", $status) if ($status < CDF_OK) ;
 
       QuitCDF ("9.1", $status) if ($varValue_out != $varValues[$x0*$DIM_1_SIZE + $x1]) ;
    }
@@ -166,7 +166,7 @@ $indices[0] = 0;
 $indices[1] = 0;
 $status = CDF::CDFvHpPut($id,$var_num,$recStart,$recCount,$recInterval,
 				\@indices,\@counts,\@intervals,\@varValues);
-QuitCDF ("10.0", $status) if ($status < &CDF_OK) ;
+QuitCDF ("10.0", $status) if ($status < CDF_OK) ;
 
 ############################################################################g
 # HyperGET from variables.
@@ -175,7 +175,7 @@ QuitCDF ("10.0", $status) if ($status < &CDF_OK) ;
 my @varBuffer_out;
 $status = CDF::CDFvHpGet($id,$var_num,$recStart,$recCount,$recInterval,
                                 \@indices,\@counts,\@intervals,\@varBuffer_out);
-QuitCDF ("11.0", $status) if ($status < &CDF_OK) ;
+QuitCDF ("11.0", $status) if ($status < CDF_OK) ;
 
 for ($x0 = 0; $x0 < $DIM_0_SIZE; $x0++) {
    for ($x1 = 0; $x1 < $DIM_1_SIZE; $x1++) {
@@ -189,10 +189,10 @@ for ($x0 = 0; $x0 < $DIM_0_SIZE; $x0++) {
 
 my $attrNum_out;
 $status = CDF::CDFattrCreate($id,$attrName, $attrScope, \$attrNum_out);
-QuitCDF ("12.0", $status) if ($status < &CDF_OK) ;
+QuitCDF ("12.0", $status) if ($status < CDF_OK) ;
 
 $status = CDF::CDFattrCreate($id,$attr2Name, $attrScope, \$attrNum_out);
-QuitCDF ("12.1", $status) if ($status < &CDF_OK) ;
+QuitCDF ("12.1", $status) if ($status < CDF_OK) ;
 
 #############################################################################
 # PUT to attributes.
@@ -201,12 +201,12 @@ my $attr_num;
 $attr_num = CDF::CDFattrNum($id,$attrName);
 $status = CDF::CDFattrPut($id,$attr_num,$entryNum,$entryDataType,
                             $entryNumElems,\$entryValue);
-QuitCDF ("13.0", $status) if ($status < &CDF_OK) ;
+QuitCDF ("13.0", $status) if ($status < CDF_OK) ;
 
 $attr_num = CDF::CDFattrNum($id,$attr2Name);
 $status = CDF::CDFattrPut($id,$attr_num,$entryNum,$entryDataType,
                             $entry1NumElems,\@entry1Values);
-QuitCDF ("13.5", $status) if ($status < &CDF_OK) ;
+QuitCDF ("13.5", $status) if ($status < CDF_OK) ;
 
 #############################################################################
 # GET from attributes.
@@ -216,7 +216,7 @@ my ($entryValue_out, @entry_out_Values);
 $attr_num = CDF::CDFattrNum($id,$attrName);
 $status = CDF::CDFattrGet($id,$attr_num,$entryNum,\$entryValue_out);
 
-QuitCDF ("14.0", $status) if ($status < &CDF_OK) ;
+QuitCDF ("14.0", $status) if ($status < CDF_OK) ;
 
 QuitCDF ("14.1", $status) if ($entryValue != $entryValue_out) ;
 $attr_num = CDF::CDFattrNum($id,$attr2Name);
@@ -231,9 +231,9 @@ for ($i=0; $i<$entry1NumElems; $i++) {
 
 my ($version_out, $release_out, $CopyRightText);
 $status = CDF::CDFdoc ($id,\$version_out,\$release_out,\$CopyRightText);
-QuitCDF ("15.0", $status) if ($status < &CDF_OK) ;
+QuitCDF ("15.0", $status) if ($status < CDF_OK) ;
 print "CDF Library Version $version_out, Release $release_out
-$CopyRightText\n";
+Copyright: $CopyRightText\n";
 
 #############################################################################
 # Inquire CDF.
@@ -244,7 +244,7 @@ my ($maxRec_out, $numRvars, $numZvars, $numAttrs_out);
 $status = CDF::CDFinquire($id,\$numDims_out,\@dimSizes_out,\$encoding_out,
 			    \$majority_out,\$maxRec_out,\$numRvars,
                             \$numAttrs_out);
-QuitCDF ("16.0", $status) if ($status < &CDF_OK) ;
+QuitCDF ("16.0", $status) if ($status < CDF_OK) ;
 
 QuitCDF ("16.1", $status) if ($numDims_out != $numDims) ;
 
@@ -263,7 +263,7 @@ QuitCDF ("16.7", $status) if ($numAttrs_out != 2) ;
 #############################################################################
 
 $status = CDF::CDFvarRename($id,$var_num,$new_varName);
-QuitCDF ("17.0a", $status) if ($status < &CDF_OK) ;
+QuitCDF ("17.0a", $status) if ($status < CDF_OK) ;
 
 #
 #############################################################################
@@ -275,7 +275,7 @@ my (@varDimVariances_out);
 $status = CDF::CDFvarInquire($id,$var_num,\$varName_out,
 			       \$varDataType_out,\$varNumElements_out,
 			       \$varRecVariance_out,\@varDimVariances_out);
-QuitCDF ("18.0", $status) if ($status < &CDF_OK) ;
+QuitCDF ("18.0", $status) if ($status < CDF_OK) ;
 
 QuitCDF ("18.1", $status) if ($varName_out ne $new_varName) ;
 QuitCDF ("18.2", $status) if ($varDataType_out != $varDataType) ;
@@ -294,14 +294,14 @@ for ($dim_n = 0; $dim_n < $numDims; $dim_n++) {
 # Close variable
 #############################################################################
 $status = CDF::CDFvarClose ($id, $var_num);
-QuitCDF ("19.0", $status) if ($status < &CDF_OK);
+QuitCDF ("19.0", $status) if ($status < CDF_OK);
 
 #############################################################################
 # Rename attribute.
 #############################################################################
 $attr_num = CDF::CDFattrNum($id,$attrName);
 $status = CDF::CDFattrRename($id,$attr_num,$new_attrName);
-QuitCDF ("20.0", $status) if ($status < &CDF_OK) ;
+QuitCDF ("20.0", $status) if ($status < CDF_OK) ;
 
 #############################################################################
 # Inquire attribute.
@@ -311,7 +311,7 @@ my ($attrName_out, $attrScope_out, $maxEntry_out);
 $attr_num = CDF::CDFattrNum($id,$new_attrName);
 $status = CDF::CDFattrInquire($id,$attr_num,\$attrName_out,\$attrScope_out,
 			        \$maxEntry_out);
-QuitCDF ("22.0", $status) if ($status < &CDF_OK) ;
+QuitCDF ("22.0", $status) if ($status < CDF_OK) ;
 
 QuitCDF ("22.1", $status) if ($attrName_out ne $new_attrName) ;
 QuitCDF ("22.2", $status) if ($attrScope_out != $attrScope) ;
@@ -325,7 +325,7 @@ QuitCDF ("22.4", $status) if ($attrNum_out != 1) ;
 my ($entryDataType_out, $entryNumElems_out);
 $status = CDF::CDFattrEntryInquire($id,$attr_num,$entryNum,
 			             \$entryDataType_out,\$entryNumElems_out);
-QuitCDF ("23.0", $status) if ($status < &CDF_OK) ;
+QuitCDF ("23.0", $status) if ($status < CDF_OK) ;
 
 QuitCDF ("23.1", $status) if ($entryDataType_out != $entryDataType) ;
 QuitCDF ("23.2", $status) if ($entryNumElems_out != $entryNumElems) ;
@@ -337,14 +337,14 @@ QuitCDF ("23.2", $status) if ($entryNumElems_out != $entryNumElems) ;
 my $errorText;
 $status = CDF::CDFerror(CDF_OK,\$errorText);
 		 
-QuitCDF ("24.0", $status) if ($status < &CDF_OK) ;
+QuitCDF ("24.0", $status) if ($status < CDF_OK) ;
 
 #############################################################################
 # Close CDF.
 #############################################################################
 
 $status = CDF::CDFclose($id);
-QuitCDF ("25.0", $status) if ($status < &CDF_OK) ;
+QuitCDF ("25.0", $status) if ($status < CDF_OK) ;
 
 #############################################################################
 # Test Epoch Routines
@@ -360,31 +360,26 @@ my $second = 3;
 my $msec = 987;
 my $epoch = CDF::computeEPOCH ($year, $month, $day, $hour, $minute, $second, $msec);
 
-my $epString0True = '13-Oct-1994 01:02:03.987';
+my $epStringTrue = '13-Oct-1994 01:02:03.987';
 my $epString1True = '19941013.0431017';
 my $epString2True = '19941013010203';
 my $epString3True = '1994-10-13T01:02:03.987Z';
 my $epString4True = '1994-10-13T01:02:03.987';
 
 my ($epString, $epString1, $epString2, $epString3, $epString4);
-#CDF::encodeEPOCH ($epoch, $epString);
-CDF::toEncodeEPOCH ($epoch, 0, $epString);
-QuitEPOCH ("30.0") if ($epString ne $epString0True);
+CDF::encodeEPOCH ($epoch, $epString);
+QuitEPOCH ("30.0") if ($epString ne $epStringTrue);
 
-#CDF::encodeEPOCH1 ($epoch, $epString1);
-CDF::toEncodeEPOCH ($epoch, 1, $epString1);
+CDF::encodeEPOCH1 ($epoch, $epString1);
 QuitEPOCH ("30.1") if ($epString1 ne $epString1True);
 
-#CDF::encodeEPOCH2 ($epoch, $epString2);
-CDF::toEncodeEPOCH ($epoch, 2, $epString2);
+CDF::encodeEPOCH2 ($epoch, $epString2);
 QuitEPOCH ("30.2") if ($epString2 ne $epString2True);
 
-#CDF::encodeEPOCH3 ($epoch, $epString3);
-CDF::toEncodeEPOCH ($epoch, 3, $epString3);
+CDF::encodeEPOCH3 ($epoch, $epString3);
 QuitEPOCH ("30.3") if ($epString3 ne $epString3True);
 
-#CDF::encodeEPOCH4 ($epoch, $epString4);
-CDF::toEncodeEPOCH ($epoch, 4, $epString4);
+CDF::encodeEPOCH4 ($epoch, $epString4);
 QuitEPOCH ("30.4") if ($epString4 ne $epString4True);
 
 my $epochOut = CDF::parseEPOCH ($epString);
@@ -413,7 +408,7 @@ my @epoch16;
 my $retco = CDF::computeEPOCH16 ($year, $month, $day, $hour, $minute, $second,
                                  $msec, $usec, $nsec, $psec, \@epoch16);
 
-my $ep16String0True = '13-Oct-1994 01:02:03.987.876.765.654';
+my $ep16StringTrue = '13-Oct-1994 01:02:03.987.876.765.654';
 my $ep16String1True = '19941013.043101711536640';
 my $ep16String2True = '19941013010203';
 my $ep16String3True = '1994-10-13T01:02:03.987.876.765.654Z';
@@ -421,25 +416,20 @@ my $ep16String4True = '1994-10-13T01:02:03.987876765654';
 
 my ($ep16String, $ep16String1, $ep16String2, $ep16String3, $ep16String4);
 
-#CDF::encodeEPOCH16 (\@epoch16, $ep16String);
-CDF::toEncodeEPOCH16 (\@epoch16, 0, $ep16String);
-QuitEPOCH ("40.0") if ($ep16String ne $ep16String0True);
+CDF::encodeEPOCH16 (\@epoch16, $ep16String);
+QuitEPOCH ("40.0") if ($ep16String ne $ep16StringTrue);
 
-#CDF::encodeEPOCH16_1 (\@epoch16, $ep16String1);
-CDF::toEncodeEPOCH16 (\@epoch16, 1, $ep16String1);
+CDF::encodeEPOCH16_1 (\@epoch16, $ep16String1);
 #print "encodeEPOCH16_1:$ep16String1";
 QuitEPOCH ("40.1") if ($ep16String1 ne $ep16String1True);
 
-#CDF::encodeEPOCH16_2 (\@epoch16, $ep16String2);
-CDF::toEncodeEPOCH16 (\@epoch16, 2, $ep16String2);
+CDF::encodeEPOCH16_2 (\@epoch16, $ep16String2);
 QuitEPOCH ("40.2") if ($ep16String2 ne $ep16String2True);
 
-#CDF::encodeEPOCH16_3 (\@epoch16, $ep16String3);
-CDF::toEncodeEPOCH16 (\@epoch16, 3, $ep16String3);
+CDF::encodeEPOCH16_3 (\@epoch16, $ep16String3);
 QuitEPOCH ("40.3") if ($ep16String3 ne $ep16String3True);
 
-#CDF::encodeEPOCH16_4 (\@epoch16, $ep16String4);
-CDF::toEncodeEPOCH16 (\@epoch16, 4, $ep16String4);
+CDF::encodeEPOCH16_4 (\@epoch16, $ep16String4);
 QuitEPOCH ("40.4") if ($ep16String4 ne $ep16String4True);
 
 my @epoch16Out;
@@ -473,28 +463,14 @@ $year = 2010;
 
 my $tt2000 = CDF::computeTT2000 ($year, $month, $day, $hour, $minute,  
                                  $second, $msec, $usec, $nsec);
-my $ttString0True = '13-Oct-2010 01:02:03.987876765';
 my $ttString3True = '2010-10-13T01:02:03.987876765';
-my $ttString4True = '2010-10-13T01:02:03.987876765Z';
-my ($ttString, $ttString3, $ttString4, $tt2000Out);
+my $ttString3;
 
-#CDF::encodeTT2000 ($tt2000, $ttString, 0);
-CDF::toEncodeTT2000 ($tt2000, 0, $ttString);
-QuitEPOCH ("50.0") if ($ttString ne $ttString0True);
-#CDF::encodeTT2000 ($tt2000, $ttString3);
-CDF::toEncodeTT2000 ($tt2000, 3, $ttString3);
-QuitEPOCH ("50.1") if ($ttString3 ne $ttString3True);
-#CDF::encodeTT2000 ($tt2000, $ttString3, 4);
-CDF::toEncodeTT2000 ($tt2000, 4, $ttString4);
-QuitEPOCH ("50.2") if ($ttString4 ne $ttString4True);
+CDF::encodeTT2000 ($tt2000, $ttString3);
+QuitEPOCH ("50.0") if ($ttString3 ne $ttString3True);
 
-#$tt2000Out = CDF::parseTT2000 ($ttString3);
-$tt2000Out = CDF::toParseTT2000 ($ttString);
+my $tt2000Out = CDF::parseTT2000 ($ttString3);
 QuitEPOCH ("51.0") if ($tt2000Out != $tt2000) ;
-$tt2000Out = CDF::toParseTT2000 ($ttString3);
-QuitEPOCH ("51.1") if ($tt2000Out != $tt2000) ;
-$tt2000Out = CDF::toParseTT2000 ($ttString4);
-QuitEPOCH ("51.2") if ($tt2000Out != $tt2000) ;
 
 my ($usecOut, $nsecOut);
 CDF::TT2000breakdown ($tt2000, $yearOut, $monthOut, $dayOut, $hourOut, 
@@ -524,15 +500,15 @@ sub QuitCDF {
 	my ($where, $status)=@_;
 
   print "Aborting at $where ...\n";
-  if ($status < &CDF_OK) {
+  if ($status < CDF_OK) {
     my $text;
-    CDF::CDFlib (&SELECT_, &CDF_STATUS_, $status,
-		   &GET_, &STATUS_TEXT_, \$text,
-		   &NULL_);
+    CDF::CDFlib (SELECT_, CDF_STATUS_, $status,
+		   GET_, STATUS_TEXT_, \$text,
+		   NULL_);
     print $text;
   }
-  CDF::CDFlib (&CLOSE_, &CDF_,
-	  &NULL_);
+  CDF::CDFlib (CLOSE_, CDF_,
+	  NULL_);
   print "...test aborted.\n";
   exit;
 	
